@@ -1,7 +1,7 @@
 /*
  * asapconv.c - converter of ASAP-supported formats
  *
- * Copyright (C) 2005-2022  Piotr Fusik
+ * Copyright (C) 2005-2025  Piotr Fusik
  *
  * This file is part of ASAP (Another Slight Atari Player),
  * see http://asap.sourceforge.net
@@ -23,6 +23,7 @@
 
 #include <stdarg.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,6 +45,7 @@
 #endif
 
 #include "asap.h"
+#include "asap-stdio.h"
 
 /* parsed command line */
 static const char *arg_output = NULL;
@@ -66,8 +68,8 @@ static void print_help(void)
 {
 	printf(
 		"Usage: asapconv [OPTIONS] INPUTFILE...\n"
-		"Each INPUTFILE must be in a supported format:\n"
-		"SAP, CMC, CM3, CMR, CMS, DMC, DLT, MPT, MPD, RMT, TMC, TM8, TM2 or FC.\n"
+		"Each INPUTFILE must be in a supported format: SAP, CMC, CM3, CMR,\n"
+		"CMS, DMC, DLT, MPT, MD1, MD2, MPD, RMT, TMC, TM8, TM2 or FC.\n"
 		"Output EXT must be one of the above or XEX, " SAMPLE_FORMATS ".\n"
 		"Options:\n"
 		"-o FILE.EXT --output=FILE.EXT  Write to the specified file\n"
@@ -167,7 +169,7 @@ static void set_music_address(const char *s)
 	arg_music_address = parse_int(s, 16, "music address", 0xffff);
 }
 
-static void apply_tags(const char *input_file, ASAPInfo *info)
+static void apply_tags(ASAPInfo *info)
 {
 	if (arg_author != NULL) {
 		if (!ASAPInfo_SetAuthor(info, arg_author))
@@ -184,23 +186,22 @@ static void apply_tags(const char *input_file, ASAPInfo *info)
 	}
 }
 
-static ASAP *load_module(const char *input_file, const unsigned char *module, int module_len)
+static ASAP *load_module(const char *input_file)
 {
 	ASAP *asap = ASAP_New();
 	if (asap == NULL)
 		fatal_error("out of memory");
 	ASAP_SetSampleRate(asap, arg_sample_rate);
-	if (!ASAP_Load(asap, input_file, module, module_len))
-		fatal_error("%s: unsupported file", input_file);
+	if (!ASAP_LoadFiles(asap, input_file, ASAPFileLoader_GetStdio()))
+		fatal_error("%s: cannot open", input_file);
 	ASAPInfo *info = (ASAPInfo *) ASAP_GetInfo(asap); /* FIXME: avoid cast */
-	apply_tags(input_file, info);
+	apply_tags(info);
 	return asap;
 }
 
-static FILE *open_output_file(const char *input_file, const unsigned char *module, int module_len, const ASAPInfo *info, bool allow_file_per_song)
+static bool get_output_file(const char *input_file, const ASAPInfo *info, bool allow_file_per_song)
 {
 	const char *output_ext = strrchr(arg_output, '.');
-	FILE *fp = NULL;
 	if (output_ext == arg_output) {
 		/* .EXT */
 	}
@@ -212,13 +213,6 @@ static FILE *open_output_file(const char *input_file, const unsigned char *modul
 	}
 	else {
 		/* FILE.EXT or -.EXT */
-		if (output_ext == arg_output + 1 && arg_output[0] == '-') {
-			/* -.EXT */
-#ifdef _WIN32
-			_setmode(_fileno(stdout), _O_BINARY);
-#endif
-			fp = stdout;
-		}
 		output_ext = NULL; /* don't insert input_file */
 	}
 
@@ -249,7 +243,7 @@ static FILE *open_output_file(const char *input_file, const unsigned char *modul
 				tag = ASAPInfo_GetDate(info);
 				break;
 			case 'e':
-				tag = ASAPInfo_GetOriginalModuleExt(info, module, module_len);
+				tag = ASAPInfo_GetOriginalModuleExt(info);
 				if (tag == NULL)
 					tag = "unknown";
 				break;
@@ -260,7 +254,7 @@ static FILE *open_output_file(const char *input_file, const unsigned char *modul
 					if (arg_song >= 0)
 						fatal_error("-s and %%s are mutually exclusive");
 					if (++current_song >= ASAPInfo_GetSongs(info))
-						return NULL;
+						return false;
 					file_per_song = true;
 				}
 				sprintf(song_tag, "%d", current_song + 1);
@@ -271,7 +265,7 @@ static FILE *open_output_file(const char *input_file, const unsigned char *modul
 				break;
 			default:
 				fatal_error("unrecognized %%%c", c);
-				return NULL;
+				return false;
 			}
 			while (*tag != '\0') {
 				if (output_ptr >= output_file + sizeof(output_file) - 1)
@@ -304,7 +298,7 @@ static FILE *open_output_file(const char *input_file, const unsigned char *modul
 
 	if (!file_per_song) {
 		if (current_song >= 0)
-			return NULL;
+			return false;
 		if (arg_song < 0)
 			current_song = ASAPInfo_GetDefaultSong(info);
 		else if (arg_song >= ASAPInfo_GetSongs(info)) {
@@ -315,15 +309,29 @@ static FILE *open_output_file(const char *input_file, const unsigned char *modul
 		else
 			current_song = arg_song;
 	}
+	return true;
+}
 
-	if (fp == NULL) {
-		fp = fopen(output_file, "wb");
-		if (fp == NULL)
-			fatal_error("cannot write %s", output_file);
+static FILE *create_file(const char *filename)
+{
+	if (arg_output[0] == '-' && arg_output[1] == '.') {
+		/* -.EXT */
+#ifdef _WIN32
+		_setmode(_fileno(stdout), _O_BINARY);
+#endif
+		return stdout;
 	}
+	FILE *fp = fopen(filename, "wb");
+	if (fp == NULL)
+		fatal_error("cannot write %s", filename);
 	return fp;
 }
 
+static FILE *open_output_file(const char *input_file, const ASAPInfo *info, bool allow_file_per_song)
+{
+	return get_output_file(input_file, info, allow_file_per_song) ? create_file(output_file) : NULL;
+}
+
 static int play_song(const char *input_file, ASAP *asap)
 {
 	int duration = arg_duration;
@@ -338,7 +346,7 @@ static int play_song(const char *input_file, ASAP *asap)
 	return duration;
 }
 
-static void write_output_file(FILE *fp, unsigned char *buffer, int n_bytes)
+static void write_output_file(FILE *fp, const uint8_t *buffer, int n_bytes)
 {
 	if (fwrite(buffer, 1, n_bytes, fp) != n_bytes) {
 		if (fp != stdout)
@@ -349,20 +357,18 @@ static void write_output_file(FILE *fp, unsigned char *buffer, int n_bytes)
 
 static void close_output_file(FILE *fp)
 {
-	if (fp != stdout) {
-		if (fclose(fp) != 0)
-			fatal_error("error closing %s", output_file);
-	}
+	if (fp != stdout && fclose(fp) != 0)
+		fatal_error("error closing %s", output_file);
 }
 
-static void convert_to_wav(const char *input_file, const unsigned char *module, int module_len, bool output_header)
+static void convert_to_wav(const char *input_file, bool output_header)
 {
-	ASAP *asap = load_module(input_file, module, module_len);
+	ASAP *asap = load_module(input_file);
 	FILE *fp;
 
-	while ((fp = open_output_file(input_file, module, module_len, ASAP_GetInfo(asap), true)) != NULL) {
+	while ((fp = open_output_file(input_file, ASAP_GetInfo(asap), true)) != NULL) {
 		play_song(input_file, asap);
-		unsigned char buffer[8192];
+		uint8_t buffer[8192];
 		int n_bytes;
 		if (output_header) {
 			n_bytes = ASAP_GetWavHeader(asap, buffer, arg_sample_format, arg_tag);
@@ -390,8 +396,8 @@ typedef int (*plame_set_num_samples)(lame_global_flags *, unsigned long);
 typedef int (*plame_set_in_samplerate)(lame_global_flags *, int);
 typedef int (*plame_set_num_channels)(lame_global_flags *, int);
 typedef int (*plame_init_params)(lame_global_flags *);
-typedef int (*plame_encode_buffer_interleaved)(lame_global_flags *, short int[], int, unsigned char *, int);
-typedef int (*plame_encode_flush)(lame_global_flags *, unsigned char *, int);
+typedef int (*plame_encode_buffer_interleaved)(lame_global_flags *, short int[], int, uint8_t *, int);
+typedef int (*plame_encode_flush)(lame_global_flags *, uint8_t *, int);
 typedef int (*plame_close)(lame_global_flags *);
 typedef int (*pid3tag_init)(lame_global_flags *);
 typedef int (*pid3tag_set_title)(lame_global_flags *, const char *);
@@ -427,9 +433,9 @@ static FARPROC lame_proc(HMODULE lame_dll, const char *name)
 
 #endif
 
-static void convert_to_mp3(const char *input_file, const unsigned char *module, int module_len)
+static void convert_to_mp3(const char *input_file)
 {
-	ASAP *asap = load_module(input_file, module, module_len);
+	ASAP *asap = load_module(input_file);
 	const ASAPInfo *info = ASAP_GetInfo(asap);
 	int channels = ASAPInfo_GetChannels(info);
 
@@ -446,7 +452,7 @@ static void convert_to_mp3(const char *input_file, const unsigned char *module,
 #endif
 
 	FILE *fp;
-	while ((fp = open_output_file(input_file, module, module_len, info, true)) != NULL) {
+	while ((fp = open_output_file(input_file, info, true)) != NULL) {
 		int duration = play_song(input_file, asap);
 		lame_global_flags *lame = lame_init();
 		if (lame == NULL)
@@ -482,9 +488,9 @@ static void convert_to_mp3(const char *input_file, const unsigned char *module,
 			id3tag_set_genre(lame, "Electronic");
 		}
 
-		unsigned char buffer[8192];
+		uint8_t buffer[8192];
 		int n_bytes;
-		unsigned char mp3buf[4096 * 5 / 4 + 7200]; /* it would be possible to reuse "buffer" instead */
+		uint8_t mp3buf[4096 * 5 / 4 + 7200]; /* it would be possible to reuse "buffer" instead */
 		int mp3_bytes;
 
 		do {
@@ -515,12 +521,26 @@ static void convert_to_mp3(const char *input_file, const unsigned char *module,
 
 #endif /* HAVE_LIBMP3LAME */
 
-static void convert_to_module(const char *input_file, const unsigned char *module, int module_len, bool output_xex)
+typedef struct {
+	bool (*save)(ASAPWriter *self, const char *filename, const uint8_t *buffer, int offset, int length);
+} ASAPWriterVtbl;
+
+static bool ASAPWriter_Save(ASAPWriter *self, const char *filename, const uint8_t *buffer, int offset, int length)
 {
-	const char *input_ext = strrchr(input_file, '.');
-	if (input_ext == NULL)
-		fatal_error("%s: missing extension", input_file);
-	input_ext++;
+	FILE *fp = create_file(filename);
+	write_output_file(fp, buffer + offset, length);
+	close_output_file(fp);
+	return true;
+}
+
+static void convert_to_module(const char *input_file, bool output_xex)
+{
+	FILE *fp = fopen(input_file, "rb");
+	if (fp == NULL)
+		fatal_error("cannot open %s", input_file);
+	static uint8_t module[ASAPInfo_MAX_MODULE_LENGTH];
+	int module_len = fread(module, 1, sizeof(module), fp);
+	fclose(fp);
 	ASAPInfo *info = ASAPInfo_New();
 	if (info == NULL)
 		fatal_error("out of memory");
@@ -529,7 +549,10 @@ static void convert_to_module(const char *input_file, const unsigned char *modul
 	ASAPWriter *writer = ASAPWriter_New();
 	if (writer == NULL)
 		fatal_error("out of memory");
-	apply_tags(input_file, info);
+	static const ASAPWriterVtbl writer_vtbl = { ASAPWriter_Save };
+	*(const ASAPWriterVtbl **) writer = &writer_vtbl;
+	ASAPWriter_SetInput(writer, input_file, ASAPFileLoader_GetStdio());
+	apply_tags(info);
 	if (arg_ntsc >= 0) {
 		if (!ASAPInfo_CanSetNtsc(info))
 			fatal_error("%s: cannot modify NTSC tag", input_file);
@@ -538,19 +561,13 @@ static void convert_to_module(const char *input_file, const unsigned char *modul
 	if (arg_music_address >= 0)
 		ASAPInfo_SetMusicAddress(info, arg_music_address);
 
-	FILE *fp;
-	while ((fp = open_output_file(input_file, module, module_len, info, output_xex)) != NULL) {
+	while (get_output_file(input_file, info, output_xex)) {
 		if (output_xex)
 			ASAPInfo_SetDefaultSong(info, current_song);
 		if (arg_duration >= 0)
 			ASAPInfo_SetDuration(info, current_song, arg_duration);
-		unsigned char output[ASAPInfo_MAX_MODULE_LENGTH];
-		ASAPWriter_SetOutput(writer, output, 0, sizeof(output));
-		int output_len = ASAPWriter_Write(writer, output_file, info, module, module_len, arg_tag);
-		if (output_len < 0)
+		if (!ASAPWriter_Write(writer, output_file, info, module, module_len, arg_tag))
 			fatal_error("%s: conversion error", input_file);
-		write_output_file(fp, output, output_len);
-		close_output_file(fp);
 	}
 
 	ASAPWriter_Delete(writer);
@@ -561,31 +578,24 @@ static void process_file(const char *input_file)
 {
 	if (arg_output == NULL)
 		fatal_error("the -o/--output option is mandatory");
-	FILE *fp = fopen(input_file, "rb");
-	if (fp == NULL)
-		fatal_error("cannot open %s", input_file);
-	static unsigned char module[ASAPInfo_MAX_MODULE_LENGTH];
-	int module_len = fread(module, 1, sizeof(module), fp);
-	fclose(fp);
-
 	const char *output_ext = strrchr(arg_output, '.');
 	if (output_ext == NULL)
 		fatal_error("missing .EXT in -o/--output");
 	output_ext++;
 	current_song = -1;
 	if (strcasecmp(output_ext, "wav") == 0)
-		convert_to_wav(input_file, module, module_len, true);
+		convert_to_wav(input_file, true);
 	else if (strcasecmp(output_ext, "raw") == 0)
-		convert_to_wav(input_file, module, module_len, false);
+		convert_to_wav(input_file, false);
 	else if (strcasecmp(output_ext, "mp3") == 0) {
 #ifdef HAVE_LIBMP3LAME
-		convert_to_mp3(input_file, module, module_len);
+		convert_to_mp3(input_file);
 #else
 		fatal_error("this build of asapconv doesn't support MP3");
 #endif
 	}
 	else
-		convert_to_module(input_file, module, module_len, strcasecmp(output_ext, "xex") == 0);
+		convert_to_module(input_file, strcasecmp(output_ext, "xex") == 0);
 }
 
 int main(int argc, char *argv[])
