@@ -1,7 +1,7 @@
 /*
  * info_dlg.c - file information dialog box
  *
- * Copyright (C) 2007-2022  Piotr Fusik
+ * Copyright (C) 2007-2025  Piotr Fusik
  *
  * This file is part of ASAP (Another Slight Atari Player),
  * see http://asap.sourceforge.net
@@ -25,7 +25,6 @@
 #include <windows.h>
 #include <stdio.h>
 #include <string.h>
-#include <tchar.h>
 #include <commctrl.h>
 
 #include "asap.h"
@@ -35,60 +34,103 @@
 #include "aatr-stdio.h"
 #endif
 
-void combineFilenameExt(LPTSTR dest, LPCTSTR filename, LPCTSTR ext)
-{
-	size_t filenameChars = _tcsrchr(filename, '.') + 1 - filename;
-	memcpy(dest, filename, filenameChars * sizeof(_TCHAR));
-	_tcscpy(dest + filenameChars, ext);
-}
+typedef struct {
+	int (*load)(const ASAPFileLoader *self, const char *filename, uint8_t *buffer, int length);
+} ASAPFileLoaderVtbl;
 
+struct ASAPFileLoader {
+	const ASAPFileLoaderVtbl *vtbl;
 #ifdef WINAMP
-LPCTSTR atrFilenameHash(LPCTSTR filename)
-{
-	for ( ; *filename != '\0'; filename++) {
-		if (_tcsnicmp(filename, _T(".atr#"), 5) == 0)
-			return filename + 4;
-	}
-	return NULL;
-}
+	AATR *disk;
 #endif
+};
 
-bool loadModule(LPCTSTR filename, BYTE *module, int *module_len)
+static int ASAPFileLoader_Load(const ASAPFileLoader *self, const char *filename, uint8_t *buffer, int length)
 {
-	bool ok;
 #ifdef WINAMP
-	LPCTSTR hash = atrFilenameHash(filename);
-	if (hash != NULL && hash < filename + MAX_PATH) {
-		_TCHAR atr_filename[MAX_PATH];
-		memcpy(atr_filename, filename, (hash - filename) * sizeof(_TCHAR));
-		atr_filename[hash - filename] = '\0';
-		ok = false;
-		AATR *disk = AATRStdio_New(atr_filename);
-		if (disk != NULL) {
-			AATRDirectory *directory = AATRDirectory_New();
-			if (directory != NULL) {
-				AATRDirectory_OpenRoot(directory, disk);
-				if (AATRDirectory_FindEntryRecursively(directory, hash + 1) && !AATRDirectory_IsEntryDirectory(directory)) {
-					AATRFileStream *stream = AATRFileStream_New();
-					if (stream != NULL) {
-						AATRFileStream_Open(stream, directory);
-						*module_len = AATRFileStream_Read(stream, module, 0, ASAPInfo_MAX_MODULE_LENGTH);
-						ok = *module_len >= 0;
-						AATRFileStream_Delete(stream);
-					}
-				}
-				AATRDirectory_Delete(directory);
+	if (self->disk != NULL) {
+		AATRDirectory *directory = AATRDirectory_New();
+		if (directory == NULL)
+			return -1;
+		AATRDirectory_OpenRoot(directory, self->disk);
+		int result = -1;
+		if (AATRDirectory_FindEntryRecursively(directory, filename) && !AATRDirectory_IsEntryDirectory(directory)) {
+			AATRFileStream *stream = AATRFileStream_New();
+			if (stream != NULL) {
+				AATRFileStream_Open(stream, directory);
+				result = AATRFileStream_Read(stream, buffer, 0, length);
+				AATRFileStream_Delete(stream);
 			}
-			AATRStdio_Delete(disk);
 		}
-		return ok;
+		AATRDirectory_Delete(directory);
+		return result;
 	}
 #endif
-	HANDLE fh = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+	HANDLE fh = CreateFileA(filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 	if (fh == INVALID_HANDLE_VALUE)
-		return false;
-	ok = ReadFile(fh, module, ASAPInfo_MAX_MODULE_LENGTH, (LPDWORD) module_len, NULL);
+		return -1;
+	BOOL ok = ReadFile(fh, buffer, length, (LPDWORD) &length, NULL);
 	CloseHandle(fh);
+	return ok ? length : -1;
+}
+
+static ASAPFileLoader *ASAPFileLoader_Open(const char **filename)
+{
+	static const ASAPFileLoaderVtbl loader_vtbl = { ASAPFileLoader_Load };
+	static ASAPFileLoader loader = { &loader_vtbl };
+#ifdef WINAMP
+	loader.disk = NULL;
+	const char *f = *filename;
+	for (int i = 0; i < MAX_PATH - 4 && f[i] != '\0'; i++) {
+		if (_strnicmp(f + i, ".atr#", 5) == 0) {
+			i += 4;
+			char atr_filename[MAX_PATH];
+			memcpy(atr_filename, f, i);
+			atr_filename[i] = '\0';
+			AATR *disk = AATRStdio_New(atr_filename);
+			if (disk == NULL)
+				return NULL;
+			*filename = f + i + 1;
+			loader.disk = disk;
+			break;
+		}
+	}
+#endif
+	return &loader;
+}
+
+static void ASAPFileLoader_Close(ASAPFileLoader *self)
+{
+#ifdef WINAMP
+	if (self->disk != NULL)
+		AATRStdio_Delete(self->disk);
+#endif
+}
+
+static bool loadInfoInternal(ASAPInfo *info, const char *filename, uint8_t *module, int *module_len)
+{
+	ASAPFileLoader *loader = ASAPFileLoader_Open(&filename);
+	if (loader == NULL)
+		return false;
+	*module_len = ASAPFileLoader_Load(loader, filename, module, ASAPInfo_MAX_MODULE_LENGTH);
+	ASAPFileLoader_Close(loader);
+	return *module_len >= 0 && ASAPInfo_Load(info, filename, module, *module_len);
+}
+
+bool loadInfo(ASAPInfo *info, const char *filename)
+{
+	uint8_t module[ASAPInfo_MAX_MODULE_LENGTH];
+	int module_len;
+	return loadInfoInternal(info, filename, module, &module_len);
+}
+
+bool loadFiles(ASAP *asap, const char *filename)
+{
+	ASAPFileLoader *loader = ASAPFileLoader_Open(&filename);
+	if (loader == NULL)
+		return false;
+	bool ok = ASAP_LoadFiles(asap, filename, loader);
+	ASAPFileLoader_Close(loader);
 	return ok;
 }
 
@@ -96,16 +138,16 @@ HWND infoDialog = NULL;
 
 #ifndef _UNICODE /* TODO */
 
-static _TCHAR playing_filename[MAX_PATH];
+static char playing_filename[MAX_PATH];
 static int playing_song = 0;
 bool playing_info = false;
-static byte saved_module[ASAPInfo_MAX_MODULE_LENGTH];
+static uint8_t saved_module[ASAPInfo_MAX_MODULE_LENGTH];
 static int saved_module_len;
 static ASAPInfo *edited_info = NULL;
 static int edited_song;
-static _TCHAR saved_author[ASAPInfo_MAX_TEXT_LENGTH + 1];
-static _TCHAR saved_title[ASAPInfo_MAX_TEXT_LENGTH + 1];
-static _TCHAR saved_date[ASAPInfo_MAX_TEXT_LENGTH + 1];
+static char saved_author[ASAPInfo_MAX_TEXT_LENGTH + 1];
+static char saved_title[ASAPInfo_MAX_TEXT_LENGTH + 1];
+static char saved_date[ASAPInfo_MAX_TEXT_LENGTH + 1];
 static bool saved_ntsc;
 static int saved_durations[ASAPInfo_MAX_SONGS];
 static bool saved_loops[ASAPInfo_MAX_SONGS];
@@ -171,7 +213,7 @@ static void updateTech(void)
 {
 	char buf[16000];
 	char *p = buf;
-	const char *ext = ASAPInfo_GetOriginalModuleExt(edited_info, saved_module, saved_module_len);
+	const char *ext = ASAPInfo_GetOriginalModuleExt(edited_info);
 	if (ext != NULL)
 		p += sprintf(p, "Composed in %s\r\n", ASAPInfo_GetExtDescription(ext));
 	int i = ASAPInfo_GetSongs(edited_info);
@@ -208,7 +250,7 @@ static void updateTech(void)
 		}
 	}
 	chomp(buf);
-	SendDlgItemMessage(infoDialog, IDC_TECHINFO, WM_SETTEXT, 0, (LPARAM) buf);
+	SendDlgItemMessageA(infoDialog, IDC_TECHINFO, WM_SETTEXT, 0, (LPARAM) buf);
 }
 
 static void updateStil(void)
@@ -248,7 +290,7 @@ static void updateStil(void)
 			return;
 		}
 	}
-	SendDlgItemMessage(infoDialog, IDC_STILINFO, WM_SETTEXT, 0, (LPARAM) buf);
+	SendDlgItemMessageA(infoDialog, IDC_STILINFO, WM_SETTEXT, 0, (LPARAM) buf);
 }
 
 static void updateTime(void)
@@ -256,7 +298,7 @@ static void updateTime(void)
 	unsigned char str[ASAPWriter_MAX_DURATION_LENGTH + 1];
 	int len = ASAPWriter_DurationToString(str, ASAPInfo_GetDuration(edited_info, edited_song));
 	str[len] = '\0';
-	SendDlgItemMessage(infoDialog, IDC_TIME, WM_SETTEXT, 0, (LPARAM) str);
+	SendDlgItemMessageA(infoDialog, IDC_TIME, WM_SETTEXT, 0, (LPARAM) str);
 }
 
 static void setEditedSong(int song)
@@ -266,10 +308,10 @@ static void setEditedSong(int song)
 	CheckDlgButton(infoDialog, IDC_LOOP, ASAPInfo_GetLoop(edited_info, song) ? BST_CHECKED : BST_UNCHECKED);
 	enableDlgItem(IDC_LOOP, ASAPInfo_GetDuration(edited_info, song) > 0);
 
-	_TCHAR filename[MAX_PATH];
-	SendDlgItemMessage(infoDialog, IDC_FILENAME, WM_GETTEXT, MAX_PATH, (LPARAM) filename);
+	char filename[MAX_PATH];
+	SendDlgItemMessageA(infoDialog, IDC_FILENAME, WM_GETTEXT, MAX_PATH, (LPARAM) filename);
 	ASTIL_Load(astil, filename, song);
-	SendDlgItemMessage(infoDialog, IDC_STILFILE, WM_SETTEXT, 0, (LPARAM) ASTIL_GetStilFilename(astil));
+	SendDlgItemMessageA(infoDialog, IDC_STILFILE, WM_SETTEXT, 0, (LPARAM) ASTIL_GetStilFilename(astil));
 	updateStil();
 }
 
@@ -279,16 +321,16 @@ static void showEditTip(int nID, LPCWSTR title, LPCWSTR message)
 	SendDlgItemMessage(infoDialog, nID, EM_SHOWBALLOONTIP, 0, (LPARAM) &ebt);
 }
 
-static bool isExt(LPCTSTR filename, LPCTSTR ext)
+static bool isExt(const char *filename, const char *ext)
 {
-	return _tcsicmp(filename + _tcslen(filename) - 4, ext) == 0;
+	return _stricmp(filename + strlen(filename) - 4, ext) == 0;
 }
 
 static void setSaved(void)
 {
-	_tcscpy(saved_author, ASAPInfo_GetAuthor(edited_info));
-	_tcscpy(saved_title, ASAPInfo_GetTitle(edited_info));
-	_tcscpy(saved_date, ASAPInfo_GetDate(edited_info));
+	strcpy(saved_author, ASAPInfo_GetAuthor(edited_info));
+	strcpy(saved_title, ASAPInfo_GetTitle(edited_info));
+	strcpy(saved_date, ASAPInfo_GetDate(edited_info));
 	saved_ntsc = ASAPInfo_IsNtsc(edited_info);
 	for (int i = 0; i < ASAPInfo_GetSongs(edited_info); i++) {
 		saved_durations[i] = ASAPInfo_GetDuration(edited_info, i);
@@ -298,9 +340,9 @@ static void setSaved(void)
 
 static bool infoChanged(void)
 {
-	if (_tcscmp(ASAPInfo_GetAuthor(edited_info), saved_author) != 0
-	 || _tcscmp(ASAPInfo_GetTitle(edited_info), saved_title) != 0
-	 || _tcscmp(ASAPInfo_GetDate(edited_info), saved_date) != 0
+	if (strcmp(ASAPInfo_GetAuthor(edited_info), saved_author) != 0
+	 || strcmp(ASAPInfo_GetTitle(edited_info), saved_title) != 0
+	 || strcmp(ASAPInfo_GetDate(edited_info), saved_date) != 0
 	 || ASAPInfo_IsNtsc(edited_info) != saved_ntsc)
 		return true;
 	for (int i = 0; i < ASAPInfo_GetSongs(edited_info); i++) {
@@ -325,10 +367,10 @@ static void updateSaveButtons(int mask, bool ok)
 	enableDlgItem(IDC_SAVEAS, ok);
 }
 
-static void updateInfoString(HWND hDlg, int nID, int mask, bool (*func)(ASAPInfo *, LPCTSTR))
+static void updateInfoString(HWND hDlg, int nID, int mask, bool (*func)(ASAPInfo *, const char *))
 {
-	_TCHAR str[ASAPInfo_MAX_TEXT_LENGTH + 1];
-	SendDlgItemMessage(hDlg, nID, WM_GETTEXT, ASAPInfo_MAX_TEXT_LENGTH + 1, (LPARAM) str);
+	char str[ASAPInfo_MAX_TEXT_LENGTH + 1];
+	SendDlgItemMessageA(hDlg, nID, WM_GETTEXT, ASAPInfo_MAX_TEXT_LENGTH + 1, (LPARAM) str);
 	bool ok = func(edited_info, str);
 	updateSaveButtons(mask, ok);
 	if (!ok)
@@ -361,7 +403,7 @@ static void toggleCalendar(HWND hDlg)
 		icex.dwSize = sizeof(icex);
 		icex.dwICC = ICC_DATE_CLASSES;
 		InitCommonControlsEx(&icex);
-		monthcal = CreateWindowEx(0, MONTHCAL_CLASS, _T(""), WS_BORDER | WS_POPUP, 0, 0, 0, 0, hDlg, NULL, NULL, NULL);
+		monthcal = CreateWindowExA(0, MONTHCAL_CLASS, "", WS_BORDER | WS_POPUP, 0, 0, 0, 0, hDlg, NULL, NULL, NULL);
 		/* subclass month calendar, so that it hides when looses focus */
 		monthcalOriginalWndProc = (WNDPROC) SetWindowLongPtr(monthcal, GWLP_WNDPROC, (LONG_PTR) MonthCalWndProc);
 	}
@@ -404,48 +446,57 @@ static void toggleCalendar(HWND hDlg)
 	}
 }
 
-static bool doSaveFile(LPCTSTR filename, bool tag)
-{
-	ASAPWriter *writer = ASAPWriter_New();
-	if (writer == NULL)
-		return false;
-	byte output[ASAPInfo_MAX_MODULE_LENGTH];
-	ASAPWriter_SetOutput(writer, output, 0, sizeof(output));
-	int output_len = ASAPWriter_Write(writer, filename, edited_info, saved_module, saved_module_len, tag);
-	ASAPWriter_Delete(writer);
-	if (output_len < 0)
-		return false;
+typedef struct {
+	bool (*save)(ASAPWriter *self, const char *filename, const uint8_t *buffer, int offset, int length);
+} ASAPWriterVtbl;
 
-	FILE *fp = _tfopen(filename, _T("wb"));
-	if (fp == NULL)
+static bool ASAPWriter_Save(ASAPWriter *self, const char *filename, const uint8_t *buffer, int offset, int length)
+{
+	HANDLE fh = CreateFileA(filename, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+	if (fh == INVALID_HANDLE_VALUE)
 		return false;
-	if (fwrite(output, output_len, 1, fp) != 1) {
-		fclose(fp);
-		DeleteFile(filename);
-		return false;
-	}
-	if (fclose(fp) != 0) {
-		DeleteFile(filename);
-		return false;
-	}
-	return true;
+	bool ok = WriteFile(fh, buffer, length, (LPDWORD) &length, NULL) != 0;
+	CloseHandle(fh);
+	return ok;
 }
 
-static bool saveFile(LPCTSTR filename, bool tag)
+static bool doSaveFile(const char *filename, bool tag)
 {
-	bool isSap = isExt(filename, _T(".sap"));
+	char sourceFilename[MAX_PATH];
+	SendDlgItemMessageA(infoDialog, IDC_FILENAME, WM_GETTEXT, MAX_PATH, (LPARAM) sourceFilename);
+	const char *sourceFilenamePtr = sourceFilename;
+	ASAPFileLoader *loader = ASAPFileLoader_Open(&sourceFilenamePtr);
+	if (loader == NULL)
+		return false;
+	ASAPWriter *writer = ASAPWriter_New();
+	if (writer == NULL) {
+		ASAPFileLoader_Close(loader);
+		return false;
+	}
+	static const ASAPWriterVtbl writer_vtbl = { ASAPWriter_Save };
+	*(const ASAPWriterVtbl **) writer = &writer_vtbl;
+	ASAPWriter_SetInput(writer, sourceFilenamePtr, loader);
+	bool ok = ASAPWriter_Write(writer, filename, edited_info, saved_module, saved_module_len, tag);
+	ASAPWriter_Delete(writer);
+	ASAPFileLoader_Close(loader);
+	return ok;
+}
+
+static bool saveFile(const char *filename, bool tag)
+{
+	bool isSap = isExt(filename, ".sap");
 	if (isSap) {
 		int song = ASAPInfo_GetSongs(edited_info);
 		while (--song >= 0 && ASAPInfo_GetDuration(edited_info, song) < 0);
 		while (--song >= 0) {
 			if (ASAPInfo_GetDuration(edited_info, song) < 0) {
-				MessageBox(infoDialog, _T("Cannot save file because time not set for all songs"), _T("Error"), MB_OK | MB_ICONERROR);
+				MessageBoxA(infoDialog, "Cannot save file because time not set for all songs", "Error", MB_OK | MB_ICONERROR);
 				return false;
 			}
 		}
 	}
 	if (!doSaveFile(filename, tag)) {
-		MessageBox(infoDialog, _T("Cannot save file"), _T("Error"), MB_OK | MB_ICONERROR);
+		MessageBoxA(infoDialog, "Cannot save file", "Error", MB_OK | MB_ICONERROR);
 		// FIXME: delete file
 		return false;
 	}
@@ -458,29 +509,29 @@ static bool saveFile(LPCTSTR filename, bool tag)
 
 static bool saveInfo(void)
 {
-	_TCHAR filename[MAX_PATH];
-	SendDlgItemMessage(infoDialog, IDC_FILENAME, WM_GETTEXT, MAX_PATH, (LPARAM) filename);
+	char filename[MAX_PATH];
+	SendDlgItemMessageA(infoDialog, IDC_FILENAME, WM_GETTEXT, MAX_PATH, (LPARAM) filename);
 	return saveFile(filename, false);
 }
 
-static void setSaveFilters(LPTSTR p)
+static void setSaveFilters(char *p)
 {
 	const char *exts[ASAPWriter_MAX_SAVE_EXTS];
-	int exts_len = ASAPWriter_GetSaveExts(exts, edited_info, saved_module, saved_module_len);
+	int exts_len = ASAPWriter_GetSaveExts(exts, edited_info);
 	for (int i = 0; i < exts_len; i++) {
 		const char *ext = exts[i];
-		p += _stprintf(p, _T("%s (*.%s)"), ASAPInfo_GetExtDescription(ext), ext) + 1;
-		p += _stprintf(p, _T("*.%s"), ext) + 1;
+		p += sprintf(p, "%s (*.%s)", ASAPInfo_GetExtDescription(ext), ext) + 1;
+		p += sprintf(p, "*.%s", ext) + 1;
 	}
 	*p = '\0';
 }
 
 static bool saveInfoAs(void)
 {
-	_TCHAR filename[MAX_PATH];
-	_TCHAR filter[1024];
-	OPENFILENAME ofn = {
-		sizeof(OPENFILENAME),
+	char filename[MAX_PATH];
+	char filter[1024];
+	OPENFILENAMEA ofn = {
+		sizeof(OPENFILENAMEA),
 		infoDialog,
 		0,
 		filter,
@@ -492,7 +543,7 @@ static bool saveInfoAs(void)
 		NULL,
 		0,
 		NULL,
-		_T("Select output file"),
+		"Select output file",
 		OFN_ENABLESIZING | OFN_EXPLORER | OFN_OVERWRITEPROMPT,
 		0,
 		0,
@@ -503,16 +554,16 @@ static bool saveInfoAs(void)
 	};
 	bool tag = false;
 	setSaveFilters(filter);
-	SendDlgItemMessage(infoDialog, IDC_FILENAME, WM_GETTEXT, MAX_PATH, (LPARAM) filename);
-	LPTSTR ext = _tcsrchr(filename, '.');
+	SendDlgItemMessageA(infoDialog, IDC_FILENAME, WM_GETTEXT, MAX_PATH, (LPARAM) filename);
+	char *ext = strrchr(filename, '.');
 	if (ext != NULL) {
 		*ext = '\0';
 		ofn.lpstrDefExt = ext + 1;
 	}
 	if (!GetSaveFileName(&ofn))
 		return false;
-	if (isExt(filename, _T(".xex"))) {
-		switch (MessageBox(infoDialog, _T("Do you want to display information during playback?"), _T("ASAP"), MB_YESNOCANCEL | MB_ICONQUESTION)) {
+	if (isExt(filename, ".xex")) {
+		switch (MessageBoxA(infoDialog, "Do you want to display information during playback?", "ASAP", MB_YESNOCANCEL | MB_ICONQUESTION)) {
 		case IDYES:
 			tag = true;
 			break;
@@ -575,7 +626,7 @@ static INT_PTR CALLBACK infoDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPAR
 		case MAKEWPARAM(IDC_TIME, EN_CHANGE):
 			{
 				char str[ASAPWriter_MAX_DURATION_LENGTH + 1];
-				SendDlgItemMessage(hDlg, IDC_TIME, WM_GETTEXT, ASAPWriter_MAX_DURATION_LENGTH + 1, (LPARAM) str);
+				SendDlgItemMessageA(hDlg, IDC_TIME, WM_GETTEXT, ASAPWriter_MAX_DURATION_LENGTH + 1, (LPARAM) str);
 				int duration = ASAPInfo_ParseDuration(str);
 				ASAPInfo_SetDuration(edited_info, edited_song, duration);
 				enableDlgItem(IDC_LOOP, str[0] != '\0');
@@ -610,7 +661,7 @@ static INT_PTR CALLBACK infoDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPAR
 			return TRUE;
 		case MAKEWPARAM(IDCANCEL, BN_CLICKED):
 			if (invalid_fields == 0 && infoChanged()) {
-				switch (MessageBox(hDlg, _T("Save changes?"), _T("ASAP"), MB_YESNOCANCEL | MB_ICONQUESTION)) {
+				switch (MessageBoxA(hDlg, "Save changes?", "ASAP", MB_YESNOCANCEL | MB_ICONQUESTION)) {
 				case IDYES:
 					if (!saveInfoAs())
 						return TRUE;
@@ -629,8 +680,8 @@ static INT_PTR CALLBACK infoDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPAR
 			LPNMSELCHANGE psc = (LPNMSELCHANGE) lParam;
 			if (psc->nmhdr.hwndFrom == monthcal && psc->nmhdr.code == MCN_SELECT) {
 				ShowWindow(monthcal, SW_HIDE);
-				_TCHAR str[32];
-				_stprintf(str, _T("%02d/%02d/%d"), psc->stSelStart.wDay, psc->stSelStart.wMonth, psc->stSelStart.wYear);
+				char str[32];
+				sprintf(str, "%02d/%02d/%d", psc->stSelStart.wDay, psc->stSelStart.wMonth, psc->stSelStart.wYear);
 				SetDlgItemText(hDlg, IDC_DATE, str);
 				ASAPInfo_SetDate(edited_info, str);
 				updateSaveButtons(INVALID_FIELD_DATE, true);
@@ -652,7 +703,7 @@ static INT_PTR CALLBACK infoDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPAR
 	return FALSE;
 }
 
-void showInfoDialog(HINSTANCE hInstance, HWND hwndParent, LPCTSTR filename, int song)
+void showInfoDialog(HINSTANCE hInstance, HWND hwndParent, const char *filename, int song)
 {
 	if (infoDialog == NULL)
 		infoDialog = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_INFO), hwndParent, infoDialogProc);
@@ -662,7 +713,7 @@ void showInfoDialog(HINSTANCE hInstance, HWND hwndParent, LPCTSTR filename, int
 		updateInfoDialog(filename, song);
 }
 
-void updateInfoDialog(LPCTSTR filename, int song)
+void updateInfoDialog(const char *filename, int song)
 {
 	if (infoDialog == NULL)
 		return;
@@ -677,25 +728,24 @@ void updateInfoDialog(LPCTSTR filename, int song)
 	}
 	else if (infoChanged())
 		return;
-	if (!loadModule(filename, saved_module, &saved_module_len)
-	 || !ASAPInfo_Load(edited_info, filename, saved_module, saved_module_len)) {
+	if (!loadInfoInternal(edited_info, filename, saved_module, &saved_module_len)) {
 		closeInfoDialog();
 		return;
 	}
 	setSaved();
-	can_save = isExt(filename, _T(".sap"));
+	can_save = isExt(filename, ".sap");
 	invalid_fields = 0;
-	SendDlgItemMessage(infoDialog, IDC_FILENAME, WM_SETTEXT, 0, (LPARAM) filename);
-	SendDlgItemMessage(infoDialog, IDC_AUTHOR, WM_SETTEXT, 0, (LPARAM) saved_author);
-	SendDlgItemMessage(infoDialog, IDC_NAME, WM_SETTEXT, 0, (LPARAM) saved_title);
-	SendDlgItemMessage(infoDialog, IDC_DATE, WM_SETTEXT, 0, (LPARAM) saved_date);
-	SendDlgItemMessage(infoDialog, IDC_SONGNO, CB_RESETCONTENT, 0, 0);
+	SendDlgItemMessageA(infoDialog, IDC_FILENAME, WM_SETTEXT, 0, (LPARAM) filename);
+	SendDlgItemMessageA(infoDialog, IDC_AUTHOR, WM_SETTEXT, 0, (LPARAM) saved_author);
+	SendDlgItemMessageA(infoDialog, IDC_NAME, WM_SETTEXT, 0, (LPARAM) saved_title);
+	SendDlgItemMessageA(infoDialog, IDC_DATE, WM_SETTEXT, 0, (LPARAM) saved_date);
+	SendDlgItemMessageA(infoDialog, IDC_SONGNO, CB_RESETCONTENT, 0, 0);
 	int songs = ASAPInfo_GetSongs(edited_info);
 	enableDlgItem(IDC_SONGNO, songs > 1);
 	for (int i = 1; i <= songs; i++) {
-		_TCHAR str[16];
-		_stprintf(str, _T("%d"), i);
-		SendDlgItemMessage(infoDialog, IDC_SONGNO, CB_ADDSTRING, 0, (LPARAM) str);
+		char str[16];
+		sprintf(str, "%d", i);
+		SendDlgItemMessageA(infoDialog, IDC_SONGNO, CB_ADDSTRING, 0, (LPARAM) str);
 	}
 	if (song < 0)
 		song = ASAPInfo_GetDefaultSong(edited_info);
@@ -707,10 +757,10 @@ void updateInfoDialog(LPCTSTR filename, int song)
 	updateTech();
 }
 
-void setPlayingSong(LPCTSTR filename, int song)
+void setPlayingSong(const char *filename, int song)
 {
-	if (filename != NULL && _tcslen(filename) < MAX_PATH - 1)
-		_tcscpy(playing_filename, filename);
+	if (filename != NULL && strlen(filename) < MAX_PATH - 1)
+		strcpy(playing_filename, filename);
 	playing_song = song;
 	if (playing_info)
 		updateInfoDialog(playing_filename, song);
